/*
** Module   : PersistentProcedure.java
** Abstract : Defines common APIs used by the PROCEDURE handle.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CA  20130112 Created initial version.
** 002 CA  20130221 Added ADM-DATa and UNIQUE-ID support.
** 003 CA  20130222 Moved here procedure-only attributes/methods from CommonProcedure.
** 004 CS  20130222 Added conversion support for CURRENT-WINDOW attribute.
** 005 CA  20130223 Added SET-CALLBACK-PROCEDURE method support.
** 006 CA  20130529 Made this resource Deletable too.
** 007 CA  20130813 Adding async request handle support: added ASYNC-REQUEST-COUNT attribute and
**                  extracted SERVER attribute in a separate interface. 
** 008 CA  20130925 Added support for PUBLISHED-EVENTS attribute.
** 009 CA  20130927 Added LegacyResource annotation, to determine resource type.
** 010 EVL 20131002 Adding legacy attributes and methods annotation.
** 011 CA  20131113 Added INSTANTIATING-PROCEDURE support.
*/

package com.goldencode.p2j.util;

import com.goldencode.p2j.ui.*;

/**
 * Exposes the following methods or attributes, common for PROCEDURE handles:
 * <table>
 * <tr>
 *    <th>attribute or method</th>
 *    <th>API</th>
 * </tr>
 * <tr>
 *    <td>INTERNAL-ENTRIES</td>
 *    <td>{@link #internalEntries()}</td>
 * </tr>
 * <tr>
 *    <td>PERSISTENT</td>
 *    <td>{@link #isPersistent()}</td>
 * </tr>
 * <tr>
 *    <td>CURRENT-WINDOW</td>
 *    <td>{@link #currentWindow()} and 
 *        {@link #setCurrentWindow}
 *    </td>
 * </tr>
 * <tr>
 *    <td>GET-SIGNATURE()</td>
 *    <td>{@link #getSignature(character)} and {@link #getSignature(String)}
 *    </td>
 * </tr>
 * <tr>
 *    <td>PROXY</td>
 *    <td>{@link #isProxy()}</td>
 * </tr>
 * <tr>
 *    <td>TRANSACTION</td>
 *    <td>{@link #getTransaction()}</td>
 * </tr>
 * <tr>
 *    <td>SET-CALLBACK-PROCEDURE</td>
 *    <td>{@link #setCallbackProcedure}</td>
 * </tr>
 * </table>
 * This interface is used as a returned type by the 
 * {@link handle#unwrapProcedure(handle)} API, which, depending on the 
 * handle's state and type, it either returns an 
 * {@link ExternalProgramWrapper} instance, or a dynamic proxy using 
 * {@link handle#invalidAttrAccessProxy}. 
 */
@LegacyResource(resource = LegacyResource.PROCEDURE)
public interface PersistentProcedure
extends CommonHandleChain,
        CommonProcedure,
        CommonFile,
        ADMData,
        UniqueID,
        Deletable,
        Remotable,
        AsyncRequestCountAttribute,
        InstantiatingProcedure
{
   /**
    * Get the <code>internal-entries</code> attribute of this procedure instance.
    * 
    * @return   See above.
    */
   @LegacyAttribute(name = "INTERNAL-ENTRIES")
   public character internalEntries();

   /**
    * Get the <code>persistent</code> attribute of this procedure instance.
    * 
    * @return   See above.
    */
   @LegacyAttribute(name = "PERSISTENT")
   public logical isPersistent();

   /**
    * Execute the <code>get-signature</code> method of this procedure instance.
    * 
    * @param    internalEntry
    *           The legacy name of the internal-entry for which the signature
    *           is needed. 
    */
   @LegacyMethod(name = "GET-SIGNATURE")
   public character getSignature(String internalEntry);

   /**
    * Execute the <code>get-signature</code> method of this procedure instance.
    * 
    * @param    internalEntry
    *           The legacy name of the internal-entry for which the signature
    *           is needed. 
    */
   @LegacyMethod(name = "GET-SIGNATURE")
   public character getSignature(character internalEntry);
   
   /**
    * Get CURRENT-WINDOW attribute for this procedure handle.
    * 
    * @return  Reference to current window widget.
    */
   @LegacyAttribute(name = "CURRENT-WINDOW")
   public WindowWidget currentWindow();

   /**
    * Set CURRENT-WINDOW attribute for this procedure handle.
    * 
    * @param    h
    *           A handle for the window.
    */
   @LegacyAttribute(name = "CURRENT-WINDOW", setter = true)
   public void setCurrentWindow(handle h);

   /**
    * Set CURRENT-WINDOW attribute for this procedure handle.
    * 
    * @param    win
    *           Window reference.
    */
   @LegacyAttribute(name = "CURRENT-WINDOW", setter = true)
   public void setCurrentWindow(WindowWidget win);
   
   /**
    * Indicates if the procedure is a proxy procedure.
    * 
    * @return   <code>true</code> if the procedure is a proxy procedure ,
    *           <code>false</code> otherwise.
    */
   @LegacyAttribute(name = "PROXY")
   public logical isProxy(); 
   
   /**
    * Returns a handle to the current opened transaction.
    * 
    * @return   See above
    */
   @LegacyAttribute(name = "TRANSACTION")
   public handle getTransaction();

   /**
    * Associate the internal procedure with the given web-services event.
    * <p>
    * This method returns false if the event name is not valid.
    * 
    * @param    event
    *           The event name, must evaluate to one of the "REQUEST-HEADER" or the 
    *           "RESPONSE-HEADER" strings. 
    * @param    proc
    *           The internal procedure to be executed for this event, which will be executed using
    *           the current THIS-PROCEDURE reference. To remove a callback mapping, set the this
    *           parameter to the empty string.
    *           
    * @return   <code>true</code> if the event was registered.
    */
   @LegacyMethod(name = "SET-CALLBACK-PROCEDURE")
   public logical setCallbackProcedure(String event, String proc);
   
   /**
    * Associate the internal procedure with the given web-services event.
    * <p>
    * This method returns false if the event name is not valid.
    * 
    * @param    event
    *           The event name, must evaluate to one of the "REQUEST-HEADER" or the 
    *           "RESPONSE-HEADER" strings. 
    * @param    proc
    *           The internal procedure to be executed for this event, which will be executed using
    *           the current THIS-PROCEDURE reference. To remove a callback mapping, set the this
    *           parameter to the empty string.
    *           
    * @return   <code>true</code> if the event was registered.
    */
   @LegacyMethod(name = "SET-CALLBACK-PROCEDURE")
   public logical setCallbackProcedure(character event, String proc);

   /**
    * Associate the internal procedure with the given web-services event.
    * <p>
    * This method returns false if the event name is not valid.
    * 
    * @param    event
    *           The event name, must evaluate to one of the "REQUEST-HEADER" or the 
    *           "RESPONSE-HEADER" strings. 
    * @param    proc
    *           The internal procedure to be executed for this event, which will be executed using
    *           the current THIS-PROCEDURE reference. To remove a callback mapping, set the this
    *           parameter to the empty string.
    *           
    * @return   <code>true</code> if the event was registered.
    */
   @LegacyMethod(name = "SET-CALLBACK-PROCEDURE")
   public logical setCallbackProcedure(String event, character proc);

   /**
    * Associate the internal procedure with the given web-services event.
    * <p>
    * This method returns false if the event name is not valid.
    * 
    * @param    event
    *           The event name, must evaluate to one of the "REQUEST-HEADER" or the 
    *           "RESPONSE-HEADER" strings. 
    * @param    proc
    *           The internal procedure to be executed for this event, which will be executed using
    *           the current THIS-PROCEDURE reference. To remove a callback mapping, set the this
    *           parameter to the empty string.
    *           
    * @return   <code>true</code> if the event was registered.
    */
   @LegacyMethod(name = "SET-CALLBACK-PROCEDURE")
   public logical setCallbackProcedure(character event, character proc);

   /**
    * Associate the internal procedure with the given web-services event.
    * <p>
    * This method returns false if the event name is not valid or if the procedure's context is
    * not a valid handle.
    * 
    * @param    event
    *           The event name, must evaluate to one of the "REQUEST-HEADER" or the 
    *           "RESPONSE-HEADER" strings. 
    * @param    proc
    *           The internal procedure to be executed for this event. To remove a callback 
    *           mapping, set the this parameter to the empty string.
    * @param    context
    *           The procedure handle where the internal procedure will be searched and executed.
    *           
    * @return   <code>true</code> if the event was registered.
    */
   @LegacyMethod(name = "SET-CALLBACK-PROCEDURE")
   public logical setCallbackProcedure(String event, String proc, handle context);

   /**
    * Associate the internal procedure with the given web-services event.
    * <p>
    * This method returns false if the event name is not valid or if the procedure's context is
    * not a valid handle.
    * 
    * @param    event
    *           The event name, must evaluate to one of the "REQUEST-HEADER" or the 
    *           "RESPONSE-HEADER" strings. 
    * @param    proc
    *           The internal procedure to be executed for this event. To remove a callback 
    *           mapping, set the this parameter to the empty string.
    * @param    context
    *           The procedure handle where the internal procedure will be searched and executed.
    *           
    * @return   <code>true</code> if the event was registered.
    */
   @LegacyMethod(name = "SET-CALLBACK-PROCEDURE")
   public logical setCallbackProcedure(character event, String proc, handle context);

   /**
    * Associate the internal procedure with the given web-services event.
    * <p>
    * This method returns false if the event name is not valid or if the procedure's context is
    * not a valid handle.
    * 
    * @param    event
    *           The event name, must evaluate to one of the "REQUEST-HEADER" or the 
    *           "RESPONSE-HEADER" strings. 
    * @param    proc
    *           The internal procedure to be executed for this event. To remove a callback 
    *           mapping, set the this parameter to the empty string.
    * @param    context
    *           The procedure handle where the internal procedure will be searched and executed.
    *           
    * @return   <code>true</code> if the event was registered.
    */
   @LegacyMethod(name = "SET-CALLBACK-PROCEDURE")
   public logical setCallbackProcedure(String event, character proc, handle context);

   /**
    * Associate the internal procedure with the given web-services event.
    * <p>
    * This method returns false if the event name is not valid or if the procedure's context is
    * not a valid handle.
    * 
    * @param    event
    *           The event name, must evaluate to one of the "REQUEST-HEADER" or the 
    *           "RESPONSE-HEADER" strings. 
    * @param    proc
    *           The internal procedure to be executed for this event. To remove a callback 
    *           mapping, set the this parameter to the empty string.
    * @param    context
    *           The procedure handle where the internal procedure will be searched and executed.
    *           
    * @return   <code>true</code> if the event was registered.
    */
   @LegacyMethod(name = "SET-CALLBACK-PROCEDURE")
   public logical setCallbackProcedure(character event, character proc, handle context);
   
   /**
    * Get the named events published by this procedure.  This is a comma separated list of
    * named events collected from the PUBLISH statements, where the named event is a hard-coded
    * string, not a variable.
    * 
    * @return    The named events published by this procedure, or the empty string if none found.
    */
   @LegacyAttribute(name = "PUBLISHED-EVENTS")
   public character getPublishedEvents();
}