/*
** Module   : SOAPFactory.java
** Abstract : Factory of SOAP objects. 
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CA  20130216 Created initial version.
** 002 CA  20130918 Added widget-pool support. The pool name must be validated before anything 
**                  else, and a null name means the default pool is used.
** 003 CA  20130927 Fixed CREATE statement calls.
** 004 CA  20131223 Added stubs to create SOAPFault and SOAPFaultDetail resources.
*/

package com.goldencode.p2j.util;

/**
 * Adds support for the {@link #createSOAPHeader CREATE SOAP-HEADER} and  
 * {@link #createSOAPHeaderEntry(handle) CREATE SOAP-HEADER-ENTRYREF} language statements. 
 */
public class SOAPFactory
{
   /**
    * Create a SOAP-HEADER object and save it in the given handle.
    * 
    * @param    h
    *           A handle variable, field or class member where the SOAP-HEADER will be saved. 
    */
   public static void createSOAPHeader(handle h)
   {
      createSOAPHeader(h, (character) null);
   }

   /**
    * Create a SOAP-HEADER object, save it in the given handle and register it in the given 
    * named widget-pool.
    * 
    * @param    h
    *           A handle variable, field or class member where the SOAP-HEADER will be saved.
    * @param    pool
    *           The name of the widget pool. <code>null</code> if the unnamed pool must be used.
    */
   public static void createSOAPHeader(handle h, String pool)
   {
      createSOAPHeader(h, (pool == null ? null : new character(pool)));
   }

   /**
    * Create a SOAP-HEADER object, save it in the given handle and register it in the given 
    * named widget-pool.
    * 
    * @param    h
    *           A handle variable, field or class member where the SOAP-HEADER will be saved.
    * @param    pool
    *           The name of the widget pool. <code>null</code> if the unnamed pool must be used.
    */
   public static void createSOAPHeader(handle h, character pool)
   {
      // validate widget pool before anything else!!!
      if (!WidgetPool.validWidgetPool(pool))
      {
         return;
      }

      WrappedResource res = new SOAPHeaderImpl();
      h.assign(res);
      
      WidgetPool.addResource(res, pool);
   }

   /**
    * Create a SOAP-HEADER-ENTRYREF object and save it in the given handle.
    * 
    * @param    h
    *           A handle variable, field or class member where the SOAP-HEADER-ENTRYREF will be 
    *           saved.
    */
   public static void createSOAPHeaderEntry(handle h)
   {
      createSOAPHeaderEntry(h, (character) null);
   }

   /**
    * Create a SOAP-HEADER-ENTRYREF object, save it in the given handle and register it in the  
    * given named widget-pool.
    * 
    * @param    h
    *           A handle variable, field or class member where the SOAP-HEADER-ENTRYREF will be 
    *           saved.
    * @param    pool
    *           The name of the widget pool. <code>null</code> if the unnamed pool must be used.
    */
   public static void createSOAPHeaderEntry(handle h, String pool)
   {
      createSOAPHeaderEntry(h, (pool == null ? null : new character(pool)));
   }


   /**
    * Create a SOAP-HEADER-ENTRYREF object, save it in the given handle and register it in the  
    * given named widget-pool.
    * 
    * @param    h
    *           A handle variable, field or class member where the SOAP-HEADER-ENTRYREF will be 
    *           saved.
    * @param    pool
    *           The name of the widget pool. <code>null</code> if the unnamed pool must be used.
    */
   public static void createSOAPHeaderEntry(handle h, character pool)
   {
      // validate widget pool before anything else!!!
      if (!WidgetPool.validWidgetPool(pool))
      {
         return;
      }

      WrappedResource res = new SOAPHeaderEntryImpl();
      h.assign(res);
      
      WidgetPool.addResource(res, pool);
   }
   
   /**
    * Create a new {@link SOAPFaultDetailImpl} resource using the given XML payload.
    * 
    * @param    payload
    *           The XML payload.
    *           
    * @return   A {@link SOAPFaultDetailImpl} for the given XML payload.
    */
   static SOAPFaultDetailImpl createSOAPFaultDetail(String payload)
   {
      // TODO: pool it?
      
      // TODO: implement me

      return null;
   }

   /**
    * Create a new {@link SOAPFaultImpl} resource.
    * 
    * @param    faultActor
    *           The SOAP fault actor.
    * @param    faultCode
    *           The SOAP fault code.
    * @param    faultString
    *           The SOAP fault string.
    * @param    soapFaultDetail
    *           The {@link SOAPFaultDetailImpl SOAP fault detail}.
    *           
    * @return   See above.
    */
   static SOAPFaultImpl createSOAPFault(String faultActor,
                                        String faultCode,
                                        String faultString,
                                        SOAPFaultDetailImpl soapFaultDetail)
   {
      // TODO: pool it?

      // TODO: implement me

      return null;
   }
}
